import json, os, re

# 파일 삭제
# 폴더 삭제
# 폴더 생성
# 현재 디렉토리 절대경로 가져오기

print(os.path.abspath('.'))

class FileMaker():
    def __init__(self, FOLDER_PATH='./Datas/', MAX_DATAS = 50000, FILE_TYPE = '.txt'):
        self.FOLDER_PATH = FOLDER_PATH
        self.MAX_DATAS = MAX_DATAS
        self.FILE_TYPE = FILE_TYPE
        self.DATA_COUNT = 0

    def Check_Folder(self, TARGET_FOLDER = ''):
        if TARGET_FOLDER:
            self.MAKE_TYPE = 0
            FOLDER_DIR = TARGET_FOLDER + '/'
            self.FILE_PATH = Create_Folder(FOLDER_DIR)
            self.FILE_NUMBER = len(File_Search(FOLDER_DIR, self.FILE_TYPE)) + 1
            return self.FILE_PATH
        else:
            self.MAKE_TYPE = 1
            self.DATA_COUNT = 0
            # 폴더 탐색 & 폴더 생성
            for li in range(65, 91):
                for si in range(65, 91):
                    FOLDER_DIR = self.FOLDER_PATH + chr(li) + chr(si) + '/'
                    try:
                        fc = len(File_Search(FOLDER_DIR, self.FILE_TYPE))
                        if fc != 100:
                            self.FILE_NUMBER = fc + 1
                            self.FILE_PATH = Create_Folder(FOLDER_DIR)
                            return self.FILE_PATH
                    except:
                        self.FILE_NUMBER = 1
                        self.FILE_PATH = Create_Folder(FOLDER_DIR)
                        return self.FILE_PATH

    def Make_File(self):
        self.FILE = open(self.FILE_PATH + str(self.FILE_NUMBER).zfill(3) + self.FILE_TYPE, 'a', -1, 'utf-8')
    
    def Write_Data(self, data, spliter='\n'):
        if self.DATA_COUNT:
            self.FILE.write(spliter + data)
        else:
            self.FILE.write(data)

        self.DATA_COUNT += 1

        if self.DATA_COUNT==self.MAX_DATAS:
            self.Close_File()
            self.DATA_COUNT = 0
            self.FILE_NUMBER += 1
            if self.MAKE_TYPE and self.FILE_NUMBER == 101:
                self.Check_Folder()
                self.FILE_NUMBER = 1
            self.Make_File()

    def Start(self):
        if self.FOLDER_PATH != './Datas/':
            self.Check_Folder(self.FOLDER_PATH)
        else:
            self.Check_Folder()
        self.Make_File()

    def Close_File(self):
        self.FILE.close()

class JsonMaker(FileMaker):
    def __init__(self, FOLDER_PATH='./Datas/', MAX_DATAS=50000):
        super().__init__(FOLDER_PATH, MAX_DATAS, FILE_TYPE='.json')
        self.FILE_TYPE='.json'

    def Make_File(self):
        super().Make_File()
        self.FILE.write('[')
    
    def Write_Data(self, data):
        if self.DATA_COUNT:
            super().Write_Data(json.dumps(data), spliter=',\n')
        else:
            self.FILE.write('\n')
            super().Write_Data(json.dumps(data))

    def Close_File(self):
        self.FILE.write('\n]')
        super().Close_File()

class CSVMaker(FileMaker):
    def __init__(self, FOLDER_PATH='./Datas/', MAX_DATAS=50000):
        super().__init__(FOLDER_PATH, MAX_DATAS, FILE_TYPE='.csv')
        self.FILE_TYPE='.csv'

    def Make_File(self, COLUMN_NAMES = ''):
        super().Make_File()
        if COLUMN_NAMES:
            self.COLUMN_NAMES = COLUMN_NAMES.split(' ')
            self.FILE.write(','.join(self.COLUMN_NAMES)+'\n')
    
    def Write_Data(self, data):
        super().Write_Data(','.join(data)+'\n')
        if self.DATA_COUNT == 0:
            self.FILE.write(','.join(self.COLUMN_NAMES)+'\n')

    def Start(self, COLUMNS_NAMES = ''):
        if self.FOLDER_PATH != './Datas/':
            self.Check_Folder(self.FOLDER_PATH)
        else:
            self.Check_Folder()
        self.Make_File(COLUMNS_NAMES)

def Create_Folder(FOLDER_DIR):
    try:
        if not os.path.exists(FOLDER_DIR):
            os.makedirs(FOLDER_DIR)
        return FOLDER_DIR
    except OSError:
        print('Error: Creating Directory' + FOLDER_DIR)

def File_Search(FOLDER_DIR, FILE_TYPE='.', ONLY_FOLDER=False):
    FILE_NAMES = os.listdir(FOLDER_DIR)
    if ONLY_FOLDER:
        return list(filter(os.path.isdir, list(map(lambda i : os.path.join(FOLDER_DIR, i), FILE_NAMES))))
    if FILE_TYPE == '.':
        return list(map(lambda i : os.path.join(FOLDER_DIR, i), FILE_NAMES))
    else:
        FILE_NAMES = list(filter(lambda i : i if FILE_TYPE in i else None, FILE_NAMES))
        return list(map(lambda i : os.path.join(FOLDER_DIR, i), FILE_NAMES))

def Change_Name(FOLDER_PATH, FILE_TYPE, TARGET_NAME, TARGET_TYPE):
    for i, name in enumerate(File_Search(FOLDER_PATH, FILE_TYPE)):
        os.rename(name, os.path.join(FOLDER_PATH, '%s_%s%s'%(TARGET_NAME, i, TARGET_TYPE)))

# 파일 삭제
# os.remove(path, option)
# 디렉토리 삭제
# os.rmdir(path, option)


import json
import random
import string

from typing import BinaryIO, Union

import requests
from requests_toolbelt import MultipartEncoder

class SessionManager:
    """
    Session Manager
    """

    history:list[requests.Response] = []

    def __init__(self, cookie_path:str='./cookie', history_mode:bool=False) -> None:
        self._headers = { 'User-Agent': 'Mozilla/5.0' }
        self._cookies = {}
        self._cookie_path = cookie_path
        self._mode = history_mode
        
        try:
            open(cookie_path, 'r')
            self._load_cookies()
        except: pass

    def __del__(self):
        self._save_cookies()
    
    # ------------------------------------------------------------------------
    # Request Management
    def get(self, url:str, headers:dict={}, cookies:dict={}, **kwargs) -> requests.Response:
        r = self._request(url, headers=headers, cookies=cookies, method='get', **kwargs)

        if self._mode:
            self.history.append(r)
        return r

    def post(self, url:str, headers:dict={}, cookies:dict={}, 
                data:Union[str, dict]={}, files:dict[str]={}, 
                no_parsing:bool=False, **kwargs) -> requests.Response:
        
        r = self._request(url, headers=headers, cookies=cookies, data=data, files=files, 
                          no_parsing=no_parsing, method='post', **kwargs)

        if self._mode:
            self.history.append(r)
        return r
    
    def put(self, url:str, headers:dict={}, cookies:dict={}, 
                 data:Union[str, dict, list]={}, files:dict[str]={}, **kwargs):
        r = self._request(url, headers=headers, cookies=cookies, data=data, files=files, method='put', **kwargs)

        if self._mode:
            self.history.append(r)
        return r

    def _request(self, url:str, headers:dict={}, cookies:dict={}, 
                 data:Union[str, dict]={}, files:dict[str]={}, 
                 method:str='post', no_parsing:bool=False, **kwargs) -> requests.Response:
        """
        request

        method = get, post, put
        files -> {key: file_path}
        """
        headers = {k.lower(): headers[k] for k in headers}

        type_header = {}
        if data and not no_parsing:
            # charset?
            type_header['content-type'] = 'application/x-www-form-urlencoded'\
                if type(data) == str else 'application/json;charset=UTF-8'

        if method.lower() == 'post':
            if files:
                boundary = '----WebKitFormBoundary' +\
                    ''.join(random.sample(string.ascii_letters + string.digits, 16))
                data = MultipartEncoder(fields={**data, **{k: self.get_file_form(v) for k, v in files.items()}}, boundary=boundary)
                headers = {**headers, 'content-type': data.content_type, "connection": "keep-alive"}

            r = requests.post(url, headers={**self._headers, **type_header, **headers}, 
                              cookies={**self._cookies, **cookies}, 
                              data=data if type(data) in [str, MultipartEncoder] or no_parsing else json.dumps(data), **kwargs)
        elif method.lower() == 'get':
            r = requests.get(url, headers={**self._headers, **type_header, **headers}, 
                             cookies={**self._cookies, **cookies}, **kwargs)
        elif method.lower() == 'put':
            r = requests.put(url, headers={**self._headers, **type_header, **headers}, 
                             cookies={**self._cookies, **cookies}, data=json.dumps(data), **kwargs)
        
        self._set_cookies(r)
        return r
    
    # ------------------------------------------------------------------------
    # Cookie Management
    def add_cookies(self, cookies:dict) -> None:
        self._cookies.update(cookies)

    def get_cookie(self, k:str) -> str:
        return self._cookies.get(k, '')

    def _set_cookies(self, r:requests.Response) -> None:
        self._cookies.update(dict(r.cookies))

    def _reset_cookies(self, keys:list=[]) -> None:
        self._cookies = {k: self._cookies[k] for k in keys}
    
    def _save_cookies(self) -> None:
        open(self._cookie_path, 'w').write('\n'.join(['{}={}'.format(k, v) for k, v in self._cookies.items()]))
        
    def _load_cookies(self) -> dict:
        for l in open(self._cookie_path, 'r').readlines():
            _idx = l.index('=')
            self._cookies.update({l[:_idx]: l[_idx+1:].rstrip()})

    # ------------------------------------------------------------------------
    # Utils
    @staticmethod
    def get_file_form(f:str) -> tuple[str, BinaryIO]:
        """
        change file path to requestable formats
        """
        fn = f.split('/').pop()
        ext = fn.split('.').pop()
        mime_type = ''
        if ext in ['jpg', 'jpeg']:
            mime_type = 'image/jpeg'
        elif ext == 'png':
            mime_type = 'image/png'
        elif ext == 'xlsx':
            mime_type = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
        elif ext == 'xls':
            mime_type = 'application/vnd.ms-excel'
        elif ext == 'zip':
            mime_type = 'application/x-zip-compressed'

        return fn, open(f, 'rb'), mime_type

if __name__ == '__main__':
    s = SessionManager()
    # s.get()
    # s.post()
    # s.put()



import os
import zipfile

def compress(file_path_names:list[tuple], zip_path:str) -> str:
    """
    compress files to .zip
    file_path_names = [
        absolute path or relative path          target_name
        ('/Users/sosin/workspace/images/1.jpg', 'my_image.jpg')
    ]
    """
    zip_file = zipfile.ZipFile(zip_path, 'w')
    for file_path, in_file_path in file_path_names:
        zip_file.write(file_path, in_file_path, compress_type=zipfile.ZIP_DEFLATED)
    zip_file.close()

    return zip_path

def extract(zip_path:str, extract_dir:str) -> str:
    """
    extract zip file to extract_dir
    """

    assert os.path.exists(extract_dir), 'extrac_dir is not exist'
    assert os.path.isdir(extract_dir), 'extract_dir is not folder'

    zip_file = zipfile.ZipFile(zip_path, 'r')
    zip_file.extractall(extract_dir)
    return extract_dir



import requests

class AligoManager:
    """
    Aligo Messages API

    https://smartsms.aligo.in/
    """

    def __init__(self, key, user_id, sender_phone, is_test) -> None:
        """
        Aligo configs
        key = API Key
        user_id = User ID
        sender_phone = phone number
        is_test = test Y/N
        """
        self.key = key 
        self.user_id = user_id 
        self.sender_phone = sender_phone 
        self.is_test = is_test 

    def send_sms(self, rec_list:list, msg_list:list, msg_type='LMS') -> int:
        """
        send messages (text)

        * required
        rec_list (receiver phone numbers)
        msg_list (messages)
        """
        mass_send_url = 'https://apis.aligo.in/send_mass/' 
        cnt = len(rec_list)
        data = {
            'key': self.key, 
            'userid': self.user_id, 
            'sender': self.sender_phone, 
            'cnt' : cnt, # 메세지 전송건수(번호,메세지 매칭건수)
            'msg_type' : msg_type, 
            'testmode_yn' : self.is_test
        }
        for i in range(1, cnt+1):
            data[f'rec_{i}'] = rec_list[i-1]
            data[f'msg_{i}'] = msg_list[i-1]

        return requests.post(mass_send_url, data=data).json()['success_cnt']
    
    def send_mms(self, rec:str, msg:str, image_path:str) -> int:
        """
        send messages (image)

        * required
        rec (receiver phone number)
        msg (message)
        image_path (image file)
        """
        send_url = 'https://apis.aligo.in/send/'
        data = {
            'key': self.key,
            'userid': self.user_id,
            'sender': self.sender_phone,
            'receiver': rec, # 수신번호 (,활용하여 1000명까지 추가 가능)
            'msg': msg, 
            'msg_type' : 'MMS',
            'testmode_yn' : self.is_test 
        }
        image_file = {'image' : open(image_path, 'rb')}

        return requests.post(send_url, data=data, files=image_file).json()['success_cnt']


class TwilioManager:
    """
    Twilio Manager

    """

    def __init__(self, account_sid:str, auth_token:str, from_number:str) -> None:
        """
        twilio config
        account_sid ACcff1...
        auth_token e1a1...
        from_number +1534...
        """
        try:
            from twilio.rest import Client
        except:
            raise ImportError('twilio module not found.\nplease enter pip install twilio')
            
        self.client = Client(account_sid, auth_token)

        self.from_ = from_number
        pass

    def send_sms(self, to:str, msg:str):
        """
        send_message
        to - receiver (+4478600..)
        msg - message
        """
        message = self.client.messages.create(
            from_ = self.from_,
            body = msg,
            to = to,
        )

        return message.sid

if __name__ == '__main__':
    config = {
        'ALIGO_KEY': 'aligo-api-key',
        'ALIGO_ID': 'aligo-user-id',
        'ALIGO_HP': 'aligo-phone-number',
        'ALIGO_TEST': 'Y/N'
    }
    aligo_mgr = AligoManager(key=config['ALIGO_KEY'], user_id=config['ALIGO_ID'], 
                             sender_phone=config['ALIGO_HP'], is_test=config['ALIGO_TEST'])
    
    receiver_list = ['00012345678', '00023456789']
    message_list = ['Test 1', 'Test 2']
    success_count = aligo_mgr.send_sms(receiver_list, message_list)
    print('success: %d'%success_count)

    success_count = aligo_mgr.send_mms('00012345678', 'Test Image', './image.jpg')
    print('success: %d'%success_count)

